<?php

/***************************************************************************

           TPLN  by HAYOUN Laurent

           website: http://tpln.sourceforge.net
           email: php_work@hotmail.com

****************************************************************************/
/*

<docgeny>
<menu name="db">
 <name lang="fr">DB fonctions</name>
 <name lang="en">DB functions</name>

 <description lang="fr">
 TPLN permet de pouvoir s'interface avec Serveurs de base de donnes via <a href="http://pear.php.net/manual/en/core.db.php" target="_blank">PEAR DB</a> :
 <br>
 <br>
 Actuellement, les bases de donnes suivantes sont supportes :
 <ul>
 <li> MySQL</li>
 <li> PostgreSQL</li>
 <li> InterBase</li>
 <li> Mini SQL</li>
 <li> Microsoft SQL Server</li>
 <li> Oracle 7/8/8i</li>
 <li> ODBC (Open Database Connectivity)</li>
 <li> SyBase</li>
 <li> Informix</li>
 <li> FrontBase</li>
 </ul>
 </description>

 <description lang="en">
 TPLN can interfaces itself with different database server via <a href="http://pear.php.net/manual/en/core.db.php">PEAR DB</a>:
 <br>
 <br>
 Currently, the following data bases are supported:
 <ul>
 <li> MySQL</li>
 <li> PostgreSQL</li>
 <li> InterBase</li>
 <li> Mini SQL</li>
 <li> Microsoft SQL Server</li>
 <li> Oracle 7/8/8i</li>
 <li> ODBC (Open Database Connectivity)</li>
 <li> SyBase</li>
 <li> Informix</li>
 <li> FrontBase</li>
 </ul>
 </description>
</menu>
*/


class TPLN_DB extends TPLN_DB_Xtra
{
 var $db_index = -1;
 var $db = array(); // objet contenant les informations de connection
 var $req; // stockage des resultats

/*
<docgeny>
<submenu name="dbconnect" parent="db">
 <name lang="fr">DbConnect</name>
 <name lang="en">DbConnect</name>

 <function>void DbConnect([string $db_type], [string $host], [string $login], [string $password], [string $base], [string $port])</function>

 <description lang="fr">
 La fonction DbConnect() sert se connecter  votre base de donnes MySQL.
 <br>
 <br>
 $db_type prend l'une des valeurs suivantes :
 <ul>
 <li>mysql  -> MySQL</li>
 <li>pgsql  -> PostgreSQL</li>
 <li>ibase  -> InterBase</li>
 <li>msql   -> Mini SQL</li>
 <li>mssql  -> Microsoft SQL Server</li>
 <li>oci8   -> Oracle 7/8/8i</li>
 <li>odbc   -> ODBC (Open Database Connectivity)</li>
 <li>sybase -> SyBase</li>
 <li>ifx    -> Informix</li>
 <li>fbsql  -> FrontBase</li>
 </ul>
 </description>

 <description lang="en">
 The DbConnect() function  serves to connect in your MySQL data base.
 <br>
 <br>
 $db_type takes one of the following values:
 <ul>
 <li>mysql  -> MySQL</li>
 <li>pgsql  -> PostgreSQL</li>
 <li>ibase  -> InterBase</li>
 <li>msql   -> Mini SQL</li>
 <li>mssql  -> Microsoft SQL Server</li>
 <li>oci8   -> Oracle 7/8/8i</li>
 <li>odbc   -> ODBC (Open Database Connectivity)</li>
 <li>sybase -> SyBase</li>
 <li>ifx    -> Informix</li>
 <li>fbsql  -> FrontBase</li>
 </ul>
 </description>

 <warning lang="fr">
 Si un argument n'est pas spcifis alors la fonction prendra les valeurs
 dfinis dans le fichier TPLN_Cfg.php.
 </warning>

 <warning lang="en">
 If an argument is not specified the function then will take the values
 definite in the TPLN_Cfg.php file.
 </warning>

 <see>DbClose</see>
</submenu>
</docgeny>
*/
 function DbConnect($db_type="",$host="",$login="",$password="",$base="",$port="")
 {

   // initialisation des variable
   if(empty($db_type)){$db_type = $GLOBALS["TPLN_DB_TYPE_DEFAULT"];}
   if(empty($host)){$host = $GLOBALS["TPLN_DB_HOST_DEFAULT"];}
   if(empty($login)){$login = $GLOBALS["TPLN_DB_LOGIN_DEFAULT"];}
   if(empty($password)){$password = $GLOBALS["TPLN_DB_PASSWORD_DEFAULT"];}
   if(empty($base)){$base = $GLOBALS["TPLN_DB_BASE_DEFAULT"];}
   if(empty($port)){$port = $GLOBALS["TPLN_DB_PORT"];}

   // avec ou sans porc ;-) ?
   //$dsn = "mysql://$user:$pass@$host/$db_name";
   if(!empty($port))
    {
      $dsn = "$db_type://$login:$password@$host:$port/$base";
    }
    else
    {
      $dsn = "$db_type://$login:$password@$host/$base";
    }


   $this->db_index++;
   $this->db[$this->db_index] =  DB::connect($dsn, true);

   if (DB::isError($this->db[$this->db_index]))
    {
      $this->_DBError(0,$this->db[$this->db_index]->getMessage());
    }

 }

/*
<docgeny>
<submenu name="changeconnection" parent="db">
 <name lang="fr">ChangeConnection</name>
 <name lang="en">ChangeConnection</name>

 <function>void ChangeConnection(int $db_index)</function>

 <description lang="fr">
 La fonction ChangeConnection() sert  changer de connection.
 </description>

 <description lang="en">
 The DoQuery() function  serves to change your connexion domain.
 </description>

 <example>
 <?php

 $TPLN = new TPLN;
 $TPLN->DbConnect(); // connect to default config (id = 0)
 $TPLN->DbConnect('ibase','server','login','root','base'); // another domain on an Interbase server ($id = 1)

 $TPLN->ChangeConnection(0);
 $TPLN->DoQuery("$my_qry"); // execute a query in first server

 $TPLN->ChangeConnection(1);
 $TPLN->DoQuery("$my_qry"); // execute a query in second server

 ?>
 </example>

 <see>DbConnect DbClose</see>
</submenu>
</docgeny>
*/
function ChangeConnection($db_index)
{
  if($db_index < 0 || $db_index >= count($this->db))
   {
     $this->_DBError('2.1',$db_index);
     return;
   }

  $this->db_index = $db_index;

}






/*
<docgeny>
<submenu name="dbclose" parent="db">
 <name lang="fr">DbClose</name>
 <name lang="en">DbClose</name>

 <function>void DbClose()</function>

 <description lang="fr">
 La fonction DbConnect() sert se dconnecter  votre base de donnes.
 </description>

 <description lang="en">
 The DbConnect() function  serves to disconnect of your database.
 </description>

 <see>DbConnect</see>
</submenu>
</docgeny>
*/
 function DbClose()
 {
 if(!is_object($this->db[$this->db_index]))
 {
   $this->_DBError(0.1);
   return;
 }

  $this->db[$this->db_index]->disconnect();
  if (DB::isError($this->db[$this->db_index]))
    {
      $this->_DBError(1,$this->db[$this->db_index]->getMessage());
    }
  }

/*
<docgeny>
<submenu name="doquery" parent="db">
 <name lang="fr">DoQuery</name>
 <name lang="en">DoQuery</name>

 <function>void DoQuery(string $query)</function>

 <description lang="fr">
 La fonction DoQuery() sert se executer une requte.
 </description>

 <description lang="en">
 The DoQuery() function  serves to execute a query.
 </description>

 <see>DbConnect DbClose</see>
</submenu>
</docgeny>
*/

 function DoQuery($query)
 {
 if(!is_object($this->db[$this->db_index]))
 {
   $this->_DBError(0.1);
   return;
 }

   $this->req = $this->db[$this->db_index]->query($query);
   if (DB::isError($this->req))
    {
      $this->_DBError(2,$this->req->getMessage());
    }
 }

/*
<docgeny>
<submenu name="getrowscount" parent="db">
 <name lang="fr">GetRowsCount</name>
 <name lang="en">GetRowsCount</name>

 <function>int GetRowsCount()</function>

 <description lang="fr">
 La fonction GetRowsCount() sert  renvoyer le nombre de rsultats d'une requte.
 </description>

 <description lang="en">
 The GetRowsCount() function serves to obtain the number of results for a query.
 </description>

 <see>DoQuery</see>
</submenu>
</docgeny>
*/
 function GetRowsCount()
 {
   if(!is_object($this->req))
    {
      $this->_DBError(2.2);
      return;
    }

   $rows_count = $this->req->numRows();
   return $rows_count;
 }

/*
<docgeny>
<submenu name="getrowscount" parent="db">
 <name lang="fr">DBNumRows</name>
 <name lang="en">DBNumRows</name>

 <function>int DBNumRows()</function>

 <description lang="fr">
 La fonction DBNumRows() sert  renvoyer le nombre de rsultats d'une requte.<br>
 Cette fonction est un alias de la fonction GetRowsCount().
 </description>

 <description lang="en">
 The DBNumRows() function serves to obtain the number of results for a query.<br>
 This function is an alias of GetRowsCount() function.
 </description>

 <see>GetRowsCount</see>
</submenu>
</docgeny>
*/
 function DBNumRows()
 {
   if(!is_object($this->req))
    {
      $this->_DBError(2.2);
      return;
    }

  $rows_count = $this->GetRowsCount();
  return $rows_count;
 }

/*
<docgeny>
<submenu name="dbfetcharray" parent="db">
 <name lang="fr">DBFetchArray</name>
 <name lang="en">DBFetchArray</name>

 <function>array DBFetchArray()</function>

 <description lang="fr">
 La fonction DBFetchArray() sert  obtenir les rsultats d'une requte sous forme d'un tableau.
 </description>

 <description lang="en">
 The DBFetchArray() function serves to obtain the results for a query in an array.
 </description>

 <example>
 <?php

 include("TPLN/TPLN.php");

 $TPLN = new TPLN();
 $TPLN->DbConnect(); // connect with default config
 $TPLN->DoQuery("SELECT * FROM samples");
 $res = $TPLN->DbFetchArray(); // result in an array
 $TPLN->DbClose();

 ?>
 </example>
</submenu>
</docgeny>
*/
 function DBFetchArray()
 {
   if(!is_object($this->req))
    {
      $this->_DBError(2.2);
      return;
    }

   $row = $this->req->fetchRow();
   return $row;
 }

/*
<docgeny>
<submenu name="dbfetchassoc" parent="db">
 <name lang="fr">DBFetchAssoc</name>
 <name lang="en">DBFetchAssoc</name>

 <function>array DBFetchAssoc()</function>

 <description lang="fr">
 La fonction DBFetchAssoc() sert  obtenir les rsultats d'une requte sous forme d'un tableau structur.
 Elle diffre dans dans le tableau cre qui possde les champs en tant que clefs dans le tableau.
 </description>

 <description lang="en">
 The DBFetchAssoc() function serves to obtain the results for a query in an structured array.
 </description>

 <example>
 <?php

 include("TPLN/TPLN.php");

 $TPLN = new TPLN();
 $TPLN->DbConnect(); // connect with default config
 $TPLN->DoQuery("SELECT * FROM samples");
 $res = $TPLN->DBFetchAssoc(); // result in a structured array
 $TPLN->DbClose();

 ?>
 </example>

 <warning lang="fr">
 Pour des raisons de performances, il est prfrable d'utilser cette fonction plutt que DBFetchArray()
 </warning>

 <warning lang="en">
 For reasons of performances, it is preferable to use this function rather than DBFetchArray()
 </warning>
</submenu>
</docgeny>

*/
function DBFetchAssoc()
 {
   if(!is_object($this->req))
    {
      $this->_DBError(2.2);
      return;
    }

   $row = $this->req->fetchRow(DB_FETCHMODE_ASSOC);
   return $row;
 }

/*
<docgeny>
<submenu name="dbfreeresult" parent="db">
 <name lang="fr">DBFreeResult</name>
 <name lang="en">DBFreeResult</name>

 <function>void DBFreeResult()</function>

 <description lang="fr">
 La fonction DBFreeResult() sert  librer la mmoire alloue  votre serveur.<br>
 Cette fonction est utile aprs de longs traitements par exemple.
 </description>

 <description lang="en">
 The DBFreeResult() function serves to free the memory allocated to your server.<br>
 This function is for example useful after long treatments.
 </description>
</submenu>
</docgeny>

*/
function DBFreeResult()
 {
  if(!is_object($this->req))
    {
      $this->_DBError(2.2);
      return;
    }

   $this->req->free();
 }


 // prise des rsulats dans un array
 function GetData($type="ASSOC")
 {
   $i = 0;

   if($type == "ASSOC")
    {
      while($row = $this->DBFetchAssoc())
          {
           $results[$i++] = $row;
          }
    }

   if($type == "ARRAY")
    {
      while($row = $this->DBFetchArray())
          {
           $results[$i++] = $row;
          }

    }

      return $results;
 }


/*
<docgeny>
<submenu name="getone" parent="db">
 <name lang="fr">GetOne</name>
 <name lang="en">GetOne</name>

 <function>string GetOne()</function>

 <description lang="fr">
 La fonction GetOne() sert  renvoyer le premier rsultat d'une requte.<br>
 Cette fonction est utile pour une requte comptage par exemple.
 </description>

 <description lang="en">
 The GetOne() function serves to obtain the first result of a query.<br>
 This function is for example useful for a 'count' query.
 </description>

 <example>
 <?php

 include("TPLN/TPLN.php");

 $TPLN = new TPLN;
 $TPLN->DbConnect();
 $TPLN->DoQuery("SELECT count(*) FROM samples");
 $count = $TPLN->GetOne();
 $TPLN->DbClose();

 ?>
 </example>

 <warning lang="fr">
 Il ne faut pas confondre GetRowsCount() avec GetOne()
 </warning>

 <warning lang="en">
 Il ne faut pas confondre GetRowsCount() avec GetOne()
 </warning>

 <see>GetRowsCount DBNumRows</see>

</submenu>
</docgeny>

*/
function GetOne()
{
  $res = $this->GetData("ARRAY");
  return $res[0][0];

}

/*
<docgeny>
<submenu name="getnumfields" parent="db">
 <name lang="fr">GetNumFields</name>
 <name lang="en">GetNumFields</name>

 <function>int GetNumFields()</function>

 <description lang="fr">
 La fonction GetNumFields() sert  renvoyer le nombre de champs de votre requte.
 </description>

 <description lang="en">
 The GetOne() function serves to obtain the number of fields of your query.
 </description>

 <example>
 <?

 include("TPLN/TPLN.php");

 $TPLN = new TPLN;
 $TPLN->DbConnect();
 $TPLN->DoQuery("SELECT * FROM samples");
 $count_fields = $TPLN->GetNumFields();
 $TPLN->DbClose();

 ?>
 </example>

 <see>GetFields ParseDBFields GetFieldName</see>

</submenu>
</docgeny>

*/
function GetNumFields()
{
   if(!is_object($this->req))
    {
      $this->_DBError(2.2);
      return;
    }

   $fields_count = $this->req->numCols();
   return $fields_count;
}

/*
<docgeny>
<submenu name="getfields" parent="db">
 <name lang="fr">GetFields</name>
 <name lang="en">GetFields</name>

 <function>array GetFields(string query) </function>

 <description lang="fr">
 La fonction GetFields() sert  renvoyer le nom des champs de votre requete.
 </description>

 <description lang="en">
 The GetFields() function serves to obtain the name of a field of your query.
 </description>

 <example>
 <?php

 include("TPLN/TPLN.php");

 $TPLN = new TPLN;
 $TPLN->DbConnect();
 $query = "SELECT * FROM table";
 $fields = $TPLN->GetFields($query); // get fields name

 for($i=0;$i<count($fields);$i++)
 {
    echo $fields[$i]."<br>";
 }
 $TPLN->DbClose();

 ?>
 </example>

 <warning lang="fr">
 TPLN 1.5
 </warning>

 <warning lang="en">
 TPLN 1.5
 </warning>

 <see>ParseDBFields</see>

</submenu>
</docgeny>
*/
function GetFields($query)
{
 $fields = array();

 // limit ?
 if(!preg_match("/LIMIT(.*)/mi",$query,$match))
 {
    $query .= " LIMIT 1";
 }


 $this->DoQuery($query);
 $row = $this->DBFetchAssoc();

 if(is_array($row))
  $fields = array_keys($row);

 return $fields;
}


/*
<docgeny>
<submenu name="getfieldname" parent="db">
 <name lang="fr">GetFieldName</name>
 <name lang="en">GetFieldName</name>

 <function>string GetFieldName(int $field_no)</function>

 <description lang="fr">
 La fonction GetFieldName() sert  renvoyer le nom de votre champs appartenant  votre requte.
 </description>

 <description lang="en">
 The GetOne() function serves to obtain the name of a field of your query.
 </description>

 <example>
 <?php

 include("TPLN/TPLN.php");

 $TPLN = new TPLN;
 $TPLN->DbConnect();
 $TPLN->DoQuery("SELECT * FROM samples");
 $count_fields = $TPLN->GetNumFields();

 for($i=0;$i<$count_fields;$i++)
 {
    echo "field $i -> ".$TPLN->GetFieldName($i)."<br>";
 }

 $TPLN->DbClose();

 ?>
 </example>

 <warning lang="fr">
 cette fonction est exprimentale, il vaut mieux utiliser GetFields()
 </warning>

 <warning lang="en">
 This is a experimental version, it's better to use GetFields()
 </warning>

 <see>GetNumFields</see>

</submenu>
</docgeny>

*/
function GetFieldName($field_no)
{
 if(!is_object($this->req))
    {
      $this->_DBError(2.2);
      return;
    }

  if($field_no < 0 || $field_no >= $this->GetNumFields())
   {
     $this->_DBError(2.3,$field_no);
     return;
   }

 // on prend la premire ligne
 $row = $this->DBFetchAssoc();
 $col = array_keys($row);

 return $col["$field_no"];

}


/*
<docgeny>
<submenu name="getdblist" parent="db">
 <name lang="fr">GetDBList</name>
 <name lang="en">GetDBList</name>

 <function>array GetDBList()</function>

 <description lang="fr">
 La fonction GetDBList() sert obtenir  le nom des bases de donnes de votre server.
 </description>

 <description lang="en">
 The GetDBList() function serves to obtain the name of databases of your server.
 </description>

 <example>
 <?

 include("TPLN/TPLN.php");

 $TPLN = new TPLN;
 $TPLN->DbConnect();
 $bases = $TPLN->GetDBList();

 for($i=0;$i<count($bases);$i++)
   {
      echo "Base {$bases[$i]}<br>";
   }

 $TPLN->DbClose();

 ?>
 </example>

</submenu>
</docgeny>

*/
function GetDBList()
{
 if(!is_object($this->db[$this->db_index]))
 {
   $this->_DBError(0.1);
   return;
 }

 $db_list = $this->db[$this->db_index]->getListOf("databases");
 return $db_list;
}


/*
<docgeny>
<submenu name="gettablelist" parent="db">
 <name lang="fr">GetTableList</name>
 <name lang="en">GetTableList</name>

 <function>array GetTableList()</function>

 <description lang="fr">
 La fonction GetTableList() sert obtenir  le nom des tables de vote base de donnes.
 </description>

 <description lang="en">
 The GetTableList() function serves to obtain the name of tables of your database.
 </description>

 <example>
 <?php

 include("TPLN/TPLN.php");

 $TPLN = new TPLN;
 $TPLN->DbConnect();
 $tables = $TPLN->GetTableList();

 for($i=0;$i<count($tables);$i++)
 {
    echo "Table {$tables[$i]}<br>";
 }

 $TPLN->DbClose();

 ?>
 </example>

</submenu>
</docgeny>

*/
function GetTableList()
{
 if(!is_object($this->db[$this->db_index]))
 {
   $this->_DBError(0.1);
   return;
 }

 $this->DoQuery("SHOW tables");

 while($row = $this->DbFetchArray())
 {
   $res[] = $row[0];
 }

 return $res;

}




}



?>